@@ -4,11 +4,9 @@ import (
 	"reflect"
 	"testing"
 
-	machinev1 "github.com/openshift/cluster-api/pkg/apis/machine/v1beta1"
-
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/aws/aws-sdk-go/service/ec2"
-
+	machinev1 "github.com/openshift/machine-api-operator/pkg/apis/machine/v1beta1"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/equality"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -163,6 +161,34 @@ func TestExtractNodeAddresses(t *testing.T) {
 				{Type: corev1.NodeHostName, Address: "ec2.example.net"},
 			},
 		},
+		{
+			testcase: "ipv6-private",
+			instance: &ec2.Instance{
+				PrivateDnsName: aws.String("ec2.example.net"),
+				NetworkInterfaces: []*ec2.InstanceNetworkInterface{
+					{
+						Status: aws.String(ec2.NetworkInterfaceStatusInUse),
+						Ipv6Addresses: []*ec2.InstanceIpv6Address{
+							{
+								Ipv6Address: aws.String("2600:1f18:4254:5100:ef8a:7b65:7782:9248"),
+							},
+						},
+						PrivateIpAddresses: []*ec2.InstancePrivateIpAddress{
+							{
+								Primary:          aws.Bool(true),
+								PrivateIpAddress: aws.String("10.0.0.5"),
+							},
+						},
+					},
+				},
+			},
+			expectedAddresses: []corev1.NodeAddress{
+				{Type: corev1.NodeInternalIP, Address: "2600:1f18:4254:5100:ef8a:7b65:7782:9248"},
+				{Type: corev1.NodeInternalIP, Address: "10.0.0.5"},
+				{Type: corev1.NodeInternalDNS, Address: "ec2.example.net"},
+				{Type: corev1.NodeHostName, Address: "ec2.example.net"},
+			},
+		},
 	}
 
 	for _, tc := range testCases {
